// producer

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdint.h>
#include "ipc.h"

int main()
{
	// Close the file in case it's already open
	shm_unlink(mmap_filename);
	// Now open file descriptor pointing to shared memory
	int fd = shm_open(mmap_filename,O_RDWR | O_CREAT, 777);//S_IRWXU | S_IRWXG | S_IRWXO);
	if (fd < 0)
	{
		perror("shm_open");
		return 0;
	}
	// Truncate it to the size of the map
	if (ftruncate(fd,szmmap) < 0)
	{
		perror("ftruncate");
		return 0;
	}
	// Memory map it so it looks just like a region of memory
	void * buf = mmap(0,szmmap,PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);

	// Shared write index
	unsigned volatile int * write_idx = (unsigned int*)((unsigned long)buf + szbuf);
	// Shared read index
	unsigned volatile int * read_idx = (unsigned int*)((unsigned long)buf + szbuf + sizeof(unsigned int));

	int i = 1024 * 64; // Transfer 64M ints
	while (1)
	{
		--i;
#ifndef NDEBUG
		printf("write:%d\t",*write_idx);
		printf("read:%d\t",*read_idx);
		printf("count:%d\t",CIRC_CNT(*write_idx, *read_idx, szbuf));
		printf("space:%d\t",CIRC_SPACE(*write_idx, *read_idx, szbuf));
#endif

		// Spin until the reader consumes something so there is space
		while(CIRC_SPACE(*write_idx,*read_idx,szbuf)<sizeof(int));

		// Write something!
		*(int*) ((unsigned long)buf+*write_idx) = i;

		// Fence it so the CPU doesn't reorder the write and the increment
		smp_wmb();

		// Increment the write pointer and modulo szbuf
		*write_idx = (*write_idx + sizeof(int)) & (szbuf - 1);
#ifndef NDEBUG
		// Print what we wrote!
		printf("i:%d\n",i);
#endif
		// If we wrote a zero then that is the signal for the reader to stop.
		if (!i) break;
	}
	munmap(buf,szmmap);
}


